"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createDelayed = exports.delay = exports.createDelayContext = void 0;
/**
 * @returns {(tag: string, timeout: number, callback: AnyFunction) => CancelDelayFunction}
 */
const createDelayContext = () => {
    const tags = {};
    const clearDelayTag = (tag) => {
        if (tags[tag] !== undefined && tags[tag] !== null) {
            clearTimeout(tags[tag]);
            tags[tag] = null;
        }
    };
    return (tag, timeout, callback) => {
        clearDelayTag(tag);
        setTimeout(() => {
            tags[tag] = null;
            callback();
        }, timeout);
        return clearDelayTag.bind(null, tag);
    };
};
exports.createDelayContext = createDelayContext;
/**
 * @type {(tag: string, timeout: number, callback: AnyFunction) => CancelDelayFunction}
 */
exports.delay = (0, exports.createDelayContext)();
/**
 * @param {AnyFunction} callback
 * @param {number} [default_threshold=-1]
 * @returns {{cancel: () => void, set: (timeout: number, set_calls_threshold?: number) => void, now: () => void}}
 */
const createDelayed = (callback, default_threshold = -1) => {
    let timeout_id = null, set_calls_made = 0;
    const cb = () => {
        set_calls_made = 0;
        callback();
    };
    return {
        set: (timeout, set_calls_threshold = default_threshold) => {
            if (set_calls_threshold > 0 && set_calls_made >= set_calls_threshold)
                return;
            if (timeout_id !== null) {
                clearTimeout(timeout_id);
                timeout_id = null;
            }
            set_calls_made++;
            timeout_id = setTimeout(cb, timeout);
        },
        cancel: () => {
            if (timeout_id !== null) {
                clearTimeout(timeout_id);
                timeout_id = null;
            }
            set_calls_made = 0;
        },
        now: () => {
            if (timeout_id !== null) {
                clearTimeout(timeout_id);
                timeout_id = null;
            }
            cb();
        }
    };
};
exports.createDelayed = createDelayed;
