import {AnyFunction, CancelDelayFunction} from "./types";

/**
 * @returns {(tag: string, timeout: number, callback: AnyFunction) => CancelDelayFunction}
 */
export const createDelayContext = () => {
    const tags: Record<string, number|null> = {};

    const clearDelayTag = (tag: string) => {
        if(tags[tag] !== undefined && tags[tag] !== null){
            clearTimeout(tags[tag]);
            tags[tag] = null;
        }
    }

    return (tag: string, timeout: number, callback: AnyFunction): CancelDelayFunction => {
        clearDelayTag(tag);

        setTimeout(() => {
            tags[tag] = null;
            callback();
        }, timeout);

        return clearDelayTag.bind(null, tag);
    };
};

/**
 * @type {(tag: string, timeout: number, callback: AnyFunction) => CancelDelayFunction}
 */
export const delay = createDelayContext();

/**
 * @param {AnyFunction} callback
 * @param {number} [default_threshold=-1]
 * @returns {{cancel: () => void, set: (timeout: number, set_calls_threshold?: number) => void, now: () => void}}
 */
export const createDelayed = (callback: AnyFunction, default_threshold: number = -1) => {
    let timeout_id: number|null = null,
        set_calls_made: number = 0;

    const cb = () => {
        set_calls_made = 0;
        callback();
    };

    return {
        set: (timeout: number, set_calls_threshold: number = default_threshold) => {
            if(set_calls_threshold > 0 && set_calls_made >= set_calls_threshold)
                return;

            if(timeout_id !== null){
                clearTimeout(timeout_id);
                timeout_id = null;
            }

            set_calls_made++;
            timeout_id = setTimeout(cb, timeout) as any;
        },
        cancel: () => {
            if(timeout_id !== null){
                clearTimeout(timeout_id);
                timeout_id = null;
            }

            set_calls_made = 0;
        },
        now: () => {
            if(timeout_id !== null){
                clearTimeout(timeout_id);
                timeout_id = null;
            }

            cb();
        }
    };
}
